-- Pixel Clock = 25.175Mhz
-- Pixel Time = 0.04 usec
--
-- HSYNC = 3.84 sec (96 Pixel), Back Porch = 1.92 sec (48 Pixel)
-- Horizontal Video = 25.6 sec (640 Pixel), Front Porch = 640 nsec (16 Pixel)
-- HSYNC + Back Porch + Horizontal Video + Front Porch = 32 s (800 Pixel)
-- HSYNC = Pixel 0 - 95
-- Back Porch = Pixel 96 - 143
-- Horizontal Video = Pixel 144 - 783
-- Front Porch = Pixel 784 - 799
--
-- VSYNC = 0,064 ms (2 horiz. lines), Back Porch = 0.928ms (29 horiz. lines)
-- Vertical Video = 15.360 sec (480 Pixel), 
-- Front Porch = 0.320 ms (10 horiz. lines)
-- VSYNC + Back Porch + Vertical Video + Front Porch = 16.672ms (521 lines)
-- VSYNC = Line 0 - 1
-- Back Porch = Line 2 - 30
-- Vertical Video = Line 31 - 510
-- Front Porch = Line 511 - 520

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VGACounter2 is
    Port ( clk :     in   STD_LOGIC;
           rst :     in   STD_LOGIC;
           HSYNC :   out  STD_LOGIC := '0';
           VSYNC :   out  STD_LOGIC := '0'; 
			  HBlank :  out  STD_LOGIC := '1';
			  VBlank :  out  STD_LOGIC := '1';
			  Shift :   out  STD_LOGIC := '0';
           VisArea : out  STD_LOGIC := '0');
end VGACounter2;

architecture Behavioral of VGACounter2 is
SIGNAL Pre_PCount :  STD_LOGIC_VECTOR(9 DOWNTO 0) := "0000000000";
SIGNAL Pre_HLCount : STD_LOGIC_VECTOR(9 Downto 0) := "0000000000";
SIGNAL Pre_HSYNC :   STD_LOGIC := '0';
SIGNAL Pre_VSYNC :   STD_LOGIC := '0';
SIGNAL HVideo :      STD_LOGIC := '0';
SIGNAL VVideo :      STD_LOGIC := '0';
begin  
  PROCESS (clk, rst, Pre_PCount, Pre_HLCount, Pre_HSYNC, Pre_VSYNC, HVideo, VVideo)
  begin
    if rising_edge(clk) then
      if rst = '0' then -- Reset is active (low)
	     HSYNC <= '0';
		  VSYNC <= '0'; 
	     Pre_HSYNC <= '0';
		  Pre_VSYNC <= '0'; 
		  Pre_PCount <= "0000000000";
		  Pre_HLCount <= "0000000000";
		  HBlank <= '1';
		  VBlank <= '1';
		  Shift <= '0';
		  HVideo <= '0';
		  VVideo <= '0';
	     VisArea <= '0'; 
        else
          Pre_PCount <= Pre_PCount + 1;
          if Pre_PCount = 800 then 
            Pre_PCount <= "0000000000";
            Pre_HLCount <= Pre_HLCount + 1;
            if Pre_HLCount = 521 then
              Pre_HLCount <= "0000000000";
		      end if;
          End if;
	   end if;
      if Pre_PCount = 0 then -- End Front Porch, Begin HSYNC
        Pre_HSYNC <= '0';
	   end if;
		if Pre_PCount = 96 then -- END HSYNC, Begin Back Porch
        Pre_HSYNC <= '1';
		end if;
      if Pre_PCount = 144 then -- End Back Porch, Begin visible Area
        HVideo <= '1';
	   end if;
      if Pre_PCount = 784 then -- End visible Area, Begin Front Porch
        HVideo <= '0';
      end if;
	   if Pre_HLCount = 0 then -- End Front Porch, Begin VSYNC
        Pre_VSYNC <= '0';
	   end if;
      if Pre_HLCount = 2 then -- END VSYNC, Begin Back Porch
        Pre_VSYNC <= '1';
	   end if;
      if Pre_HLCount = 31 then -- End Back Porch, Begin visible Area
        VVideo <= '1';
	   end if;
      if Pre_HLCount = 511 then -- End visible Area, Begin Front Porch
        VVideo <= '0';
	   end if;		
      HSYNC <= Pre_HSYNC;
      VSYNC <= Pre_VSYNC;
      if Pre_PCount = 784 then
        HBlank <= '1';
      end if;
      if Pre_PCount = 144 then
        HBlank <= '0';
      end if;
      if Pre_HLCount = 511 then
        VBlank <= '1';
	   end if;
      if Pre_HLCount = 31 then
        VBlank <= '0';
	   end if;
      if Pre_PCount = 142 then
        Shift <= '1'; -- Means SDRAM BURST START
      end if;  
      if Pre_PCount = 781 then
        Shift <= '0'; -- Means SDRAM WRITE START
      end if;
	   if HVideo = '1' and VVideo = '1' then -- X & Y Pixelposition is in visible Area
        VisArea <= '1';
        else
          VisArea <= '0';
	   end if;
    end if;
  END PROCESS;
end Behavioral;